@@ -28,6 +28,7 @@ import jsonpickle
 import copy
 import threading
 from BPTK_Py.externalstateadapter import InstanceState, ExternalStateAdapter
+from functools import wraps
 
 class InstanceManager:
     """
@@ -190,7 +191,7 @@ class BptkServer(Flask):
     """
     This class provides a Flask-based server that provides a REST-API for running bptk scenarios. The class inherts the properties and methods of Flask and doesn't expose any further public methods.
     """
-    def __init__(self, import_name, bptk_factory=None, external_state_adapter=None):
+    def __init__(self, import_name, bptk_factory=None, external_state_adapter=None, authenticate = False, bearer_token=None):
         """
         Initialize the server with the import name and the bptk.
         :param import_name: the name of the application package. Usually __name__. This helps locate the root_path for the blueprint.
@@ -200,6 +201,8 @@ class BptkServer(Flask):
         self._bptk = bptk_factory() if bptk_factory is not None else None
         self._external_state_adapter = external_state_adapter
         self._instance_manager = InstanceManager(bptk_factory)
+        self._bearer_token = bearer_token		
+        self._authenticate = authenticate
 
         # Loading the full state on startup
         if external_state_adapter != None:
@@ -228,6 +231,26 @@ class BptkServer(Flask):
         self.route("/load-state", methods=['GET'], strict_slashes=False)(self._load_state_resource)
         self.route("/<instance_uuid>/stop-instance", methods=['GET'], strict_slashes=False)(self._stop_instance)
 
+    def token_required(f):
+        @wraps(f)
+        def decorated(self, *args, **kwargs):
+            if self._authenticate:
+                token = None
+                if "Authorization" in request.headers:
+                    token = request.headers["Authorization"].split(" ")[1]
+
+                if token is None:
+                    resp = make_response('{"Unauthorized": "Authentication Token is missing!"}', 401)
+                    return resp
+                
+                if token != self._bearer_token:
+                    resp = make_response('{"Unauthorized": "Authentication Token is wrong!"}', 401)
+                    return resp
+
+            return f(self, *args, **kwargs)
+        return decorated
+
+    @token_required
     def _stop_instance(self, instance_uuid):
         self._instance_manager._delete_instance(instance_uuid)
         if self._external_state_adapter != None:
@@ -238,6 +261,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _save_state_resource(self):
         """
         Save all instances with the provided external state adapter.
@@ -253,6 +277,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
     
+    @token_required
     def _load_state_resource(self):
         """
         Loads all instances using the external state adapter
@@ -270,6 +295,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _metrics(self):
         """
         Returns metrics in a prometheus compatible format.
@@ -278,6 +304,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _full_metrics(self):
         """
         Returns metrics in JSON format. Following metrics are returned:
@@ -289,12 +316,14 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _home_resource(self):
         """
         The root endpoint returns a simple html page for test purposes.
         """
         return "<h1>BPTK-Py REST API Server</h1>"
 
+    @token_required
     def _run_resource(self):
         """
         Given a JSON dictionary that defines the relevant simulation scenarios and equations, this endpoint runs those scenarios and returns the data generated by the simulations.
@@ -365,6 +394,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _scenarios_resource(self):
         """
         The endpoint returns all available scenarios for the current simulation.
@@ -382,6 +412,7 @@ class BptkServer(Flask):
 
         return resp
 
+    @token_required
     def _equations_resource(self):
         """
         This endpoint returns all available equations given the name of a scenario manager and of a scenario.
@@ -447,6 +478,7 @@ class BptkServer(Flask):
 
         return resp
 
+    @token_required
     def _agents_resource(self):
         """
         For an agent-based or hybrid model, this endpoint returns all the agents in the model with their corresponding states and properties.
@@ -500,6 +532,7 @@ class BptkServer(Flask):
 
         return resp
 
+    @token_required
     def _start_instance_resource(self):
         """
         This endpoint starts a new instance of BPTK on the server side, so that simulations can run in a "private" session. The endpoint returns an instance_id, which is needed to identify the instance in later calls.
@@ -525,6 +558,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _begin_session_resource(self, instance_uuid):
         """This endpoint starts a session for single step simulation. There can only be one session per instance at a time.
         Currently only System Dynamics scenarios are supported for both SD DSL and XMILE models.
@@ -605,6 +639,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _end_session_resource(self, instance_uuid):
         """This endpoint ends a session for single step simulation and resets the internal cache.
         """
@@ -623,7 +658,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
-
+    @token_required
     def _flat_session_results_resource(self,instance_uuid):
         """
         Returns the accumulated results of a session, from the first step to the last step that was run in a flat format.
@@ -636,6 +671,7 @@ class BptkServer(Flask):
 
         return self._session_results_resource(instance_uuid, True)
 
+    @token_required
     def _session_results_resource(self,instance_uuid,flat=False):
         """
         Returns the accumulated results of a session, from the first step to the last step that was run.
@@ -655,7 +691,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin'] = '*'
         return resp
 
-
+    @token_required
     def _run_step_resource(self, instance_uuid):
         """
         This endpoint advances the relevant scenarios by one timestep and returns the data for that timestep.
@@ -703,6 +739,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _run_steps_resource(self, instance_uuid):
         """
         This endpoint advances the relevant scenarios by one timestep and returns the data for that timestep.
@@ -764,7 +801,7 @@ class BptkServer(Flask):
         return resp
 
 
-
+    @token_required
     def _stream_steps_resource(self, instance_uuid):
         """
         This endpoint is used to stream a simulation.
@@ -829,7 +866,7 @@ class BptkServer(Flask):
         return resp
 
 
-
+    @token_required
     def _keep_alive_resource(self,instance_uuid):
         """
         This endpoint sets the "last accessed time" of the instance to the current time to prevent the instance from timeing out.
@@ -847,6 +884,7 @@ class BptkServer(Flask):
         resp.headers['Access-Control-Allow-Origin']='*'
         return resp
 
+    @token_required
     def _ensure_instance_exists(self, instance_uuid) -> bool:
         if self._instance_manager.is_valid_instance(instance_uuid):
             return True
